==> blcr-0.5.0.tar.gz: blcr-0.5.0/tests/stage0002.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: stage0002.c,v 1.3 2004/12/09 19:14:54 phargrov Exp $
 */

const char description[] =
"Description of tests/Stage0002/cr_test:\n"
"\n"
"This test verifies the basic 'Stage II' functionality of both libcr and the\n"
"blcr.o kernel module.  In addition to 'Stage I', this includes the following:\n"
" + cr_register_callback()\n"
"        Registers an thread context callback.\n"
" + cr_{enter,leave}_cs(CR_ID_CALLBACK) does not block in callbacks.\n"
" + Externally initiated checkpoint (runs cr_checkpoint utility).\n"
" + Ability to fork() and waitpid() from the thread-context callback.\n"
;

#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include "libcr.h"

static volatile int flag = 0;

static int my_thread_callback(void *arg)
{
    pid_t child;

    cr_enter_cs(CR_ID_CALLBACK);
    printf("002 thread-context callback running in pid %d\n", (int)getpid());
    child = fork();
    if (child > 0) {
	waitpid(child, NULL, 0);
    } else if (child == 0) {
	printf("003 in fork()ed child pid=%d\n", (int)getpid());
	sleep(1);
	exit(0);
    } else {
	printf("XXX thread-context callback failed to fork()\n");
	exit(-1);
    }
    printf("004 thread-context callback complete\n");
    cr_leave_cs(CR_ID_CALLBACK);

    return 0;
}

static int my_signal_callback(void *arg)
{
    flag = 1;
    return 0;
}

static int child_main(void)
{
    pid_t child;
    struct timeval tv;
    
    setlinebuf(stderr);

    (void)cr_init();
    (void)cr_register_callback(my_signal_callback, NULL, CR_SIGNAL_CONTEXT);
    (void)cr_register_callback(my_thread_callback, NULL, CR_THREAD_CONTEXT);

    printf("000 First process started with pid %d\n", getpid());

    child = fork();
    if (child > 0) {
	printf("001 Second process started with pid %d\n", child);
    } else if (child < 0) {
	printf("XXX First process failed to fork\n");
	exit(-1);
    }

    tv.tv_sec  = 5;
    tv.tv_usec = 0;
    while (tv.tv_sec || tv.tv_usec) {
	select(0, NULL, NULL, NULL, &tv);
    }

    if (!child) {
	printf("005 Exiting from pid %d\n", getpid());
    } else {
	waitpid(child, NULL, 0);
	if (flag) {
	    printf("006 verified that signal-context callback ran\n");
	}
	printf("007 Exiting from pid %d\n", getpid());
    }

    return 0;
}

int main(int argc, char **argv)
{
    char buf[1024];
    pid_t child;
    struct stat s;
    int rc;

    setlinebuf(stdout);

    child = fork();

    if (child == 0) {
	return child_main();
    } else if (child < 0) {
	printf("XXX main failed to fork()\n");
	exit(-1);
    }


    sleep(1);

    snprintf(buf, 1024, TOPDIR "/util/cr_checkpoint/cr_checkpoint -f context.%d %d 2>/dev/null", child, child);
    rc = system(buf);
    if (rc) {
	printf("XXX got %d from system(%s)\n", rc, buf);
	exit(-1);
    }

    waitpid(child, NULL, 0);

    snprintf(buf, 1024, "context.%d", child);
    rc = stat(buf, &s);
    if (rc < 0) {
	printf("XXX failed to stat() %s\n", buf);
	exit(-1);
    } else if (s.st_size == 0) {
	printf("XXX failed to stat() %s\n", buf);
	exit(-1);
    } else {
	printf("008 %s has size %lu\n", buf, (unsigned long)s.st_size);
    }

    (void)unlink(buf); // might fail silently

    printf("009 DONE\n");

    return 0;
}

==> blcr-0.6.2.tar.gz: blcr-0.6.2/tests/stage0002.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: stage0002.c,v 1.7 2007/07/21 03:03:31 phargrov Exp $
 */

const char description[] =
"Description of tests/Stage0002/cr_test:\n"
"\n"
"This test verifies the basic 'Stage II' functionality of both libcr and the\n"
"blcr.o kernel module.  In addition to 'Stage I', this includes the following:\n"
" + cr_register_callback()\n"
"        Registers an thread context callback.\n"
" + cr_{enter,leave}_cs(CR_ID_CALLBACK) does not block in callbacks.\n"
" + Externally initiated checkpoint (runs cr_checkpoint utility).\n"
" + Ability to fork() and waitpid() from the thread-context callback.\n"
;

#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include "libcr.h"
#include "crut_util.h"

static volatile int flag = 0;

static int my_thread_callback(void *arg)
{
    pid_t child;

    cr_enter_cs(CR_ID_CALLBACK);
    printf("002 thread-context callback running in pid %d\n", (int)getpid());
    child = fork();
    if (child > 0) {
	waitpid(child, NULL, 0);
    } else if (child == 0) {
	printf("003 in fork()ed child pid=%d\n", (int)getpid());
	sleep(1);
	exit(0);
    } else {
	printf("XXX thread-context callback failed to fork()\n");
	exit(-1);
    }
    printf("004 thread-context callback complete\n");
    cr_leave_cs(CR_ID_CALLBACK);

    return 0;
}

static int my_signal_callback(void *arg)
{
    flag = 1;
    return 0;
}

static int child_main(void)
{
    pid_t child;
    struct timeval tv;
    
    setlinebuf(stderr);

    (void)cr_init();
    (void)cr_register_callback(my_signal_callback, NULL, CR_SIGNAL_CONTEXT);
    (void)cr_register_callback(my_thread_callback, NULL, CR_THREAD_CONTEXT);

    printf("000 First process started with pid %d\n", getpid());
    printf("#ST_ALARM:120\n");

    child = fork();
    if (child > 0) {
	printf("001 Second process started with pid %d\n", child);
    } else if (child < 0) {
	printf("XXX First process failed to fork\n");
	exit(-1);
    }

    tv.tv_sec  = 5;
    tv.tv_usec = 0;
    while (tv.tv_sec || tv.tv_usec) {
	select(0, NULL, NULL, NULL, &tv);
    }

    if (!child) {
	printf("005 Exiting from pid %d\n", getpid());
    } else {
	waitpid(child, NULL, 0);
	if (flag) {
	    printf("006 verified that signal-context callback ran\n");
	}
	printf("007 Exiting from pid %d\n", getpid());
    }

    return 0;
}

int main(int argc, char **argv)
{
    char buf[1024];
    pid_t child;
    struct stat s;
    int rc;

    setlinebuf(stdout);

    child = fork();

    if (child == 0) {
	return child_main();
    } else if (child < 0) {
	printf("XXX main failed to fork()\n");
	exit(-1);
    }


    sleep(1);

    snprintf(buf, 1024, "%s -f context.%d %d 2>/dev/null",
		crut_find_cmd("cr_checkpoint"), child, child);
    rc = system(buf);
    if (rc) {
	printf("XXX got %d from system(%s)\n", rc, buf);
	exit(-1);
    }

    waitpid(child, NULL, 0);

    snprintf(buf, 1024, "context.%d", child);
    rc = stat(buf, &s);
    if (rc < 0) {
	printf("XXX failed to stat() %s\n", buf);
	exit(-1);
    } else if (s.st_size == 0) {
	printf("XXX failed to stat() %s\n", buf);
	exit(-1);
    } else {
	printf("008 %s has size %lu\n", buf, (unsigned long)s.st_size);
    }

    (void)unlink(buf); // might fail silently

    printf("009 DONE\n");

    return 0;
}
